function make_frame(istp,field,save)
%Plots timestep istp of FLAC run
%field can be 'Strain', 'Temperature' or 'Phase'

%==========================================================================
%save image if save = true (false by default)
if nargin > 2
    save = save;
else
    save = false;
end

xmin = -30;
xmax = 30;
zmin = -20;
zmax = 4;
rho = 3300;
alpha = 3e-5;

%==========================================================================
%Loading the different fields
if exist('platform.s','file') == 0
  plat = 'native';
elseif exist('platform.s','file') == 2
  plat = 'ieee-le';
end
fid_t = fopen('time.0','r',plat);
fid1 = fopen('sxx.0','r',plat);
fid2 = fopen('syy.0','r',plat);
fid3 = fopen('aps.0','r',plat);
fid4 = fopen('aps.0','r',plat);
fid5 = fopen('temp.0','r',plat);
fid6 = fopen('phas.0','r',plat);

loadflac;

tsp = round(t(istp)/3600/24/365/1000);

%==========================================================================
for ii = 1:length(t)
    for jj = 1:nx
        %dZ = abs(diff([reshape(meshzz(1:end-1,jj,ii),ny,1);meshzz(1,jj,ii)+meshzz(end,1,1)]));
        dZ = -(diff([reshape(meshzz(1:end-1,jj,ii),ny,1);meshzz(1,jj,ii)+meshzz(end,1,1)]));
        drho = reshape(rho*(1-(alpha)*A4(:,jj,ii)),1,ny);
        tmass(jj) = drho*dZ*1e3;
    end
    zbox = meshzz(1,1,1)-meshzz(end,1,1);
    if tmass==0
        ziso(ii,:) = zeros(size(tmass));
    else
        ziso(ii,:) = zbox./(tmass./tmass(1))-zbox;
    end
end

kt=istp;
a1=zeros(ny,nx); 

%==========================================================================
%Computing arrays
if istp > 3
    ma1 = (A1(:,:,kt-3) + A1(:,:,kt-2) + A1(:,:,kt-1) + A1(:,:,kt))./4;
    ma3 = (A3(:,:,kt-3) + A3(:,:,kt-2) + A3(:,:,kt-1) + A3(:,:,kt))./4;
    ma4 = (A4(:,:,kt-3) + A4(:,:,kt-2) + A4(:,:,kt-1) + A4(:,:,kt))./4;
    ma6 = (A6(:,:,kt-3) + A6(:,:,kt-2) + A6(:,:,kt-1) + A6(:,:,kt))./4;
else
    ma1=A1(:,:,kt);ma3=A3(:,:,kt);ma4=A4(:,:,kt);ma6=A6(:,:,kt);
end
ma5 = A5(:,:,kt);
xcnr=zeros(ny*nx,4); zcnr=xcnr;
xcnr(:)=Xcnr(:,:,kt);  zcnr(:)=Zcnr(:,:,kt);

vfac=6;qsize=1;
jv=[1:vfac:nx]; iv=[1:vfac:ny];

Vx=zeros([length(iv) length(jv)]); Vy=Vx; Xv=Vx; Yv=Vx;
Vx(:)=vx(iv,jv,kt); Vy(:)=vz(iv,jv,kt);
Xv(:)=meshxx(iv,jv,kt); Yv(:)=meshzz(iv,jv,kt);

%==========================================================================
%Plotting the selected field

if strcmp(field,'Strain')
    ma = ma4;
elseif strcmp(field,'Temperature')
    ma = ma5;
elseif strcmp(field,'Phase')
    ma = ma6;
end

patch(xcnr',zcnr',reshape(ma',length(xcnr),1)','edgecolor','none')
hold on

%==========================================================================
%Adjusting colors

if strcmp(field,'Strain')
    caxis([0 0.5])
elseif strcmp(field,'Temperature')
    caxis([0 1300])
elseif strcmp(field,'Phase')
    caxis([1 4])
end
%==========================================================================
%Velocity field
%quiver(Xv,Yv,Vx,Vy,qsize,'Color',[.4 .4 .4]);

%==========================================================================
%Plotting topography
%plot(meshxx(1,:,kt),meshzz(1,:,kt),'g','linewidth',2.5)

%==========================================================================
%Plotting the Moho and 600°C and 1200°C isotherms
for i=1:180
    Xtherm(i)=-30+(i-1)*1/3;
end
for i=1:55
    Ztherm(i)=-(i-1)*20/55;
end
ma6(1:17,:)=1;
[C,h]=contour(Xtherm,Ztherm,ma5,[600 1200],'w','ShowText','on','LineWidth',2);
clabel(C,h,'FontSize',15,'Color','white','LabelSpacing',1200);
[C2,h2]=contour(Xtherm,Ztherm,ma6,[2.5 2.5],'r','LineWidth',2);

%==========================================================================
%Plotting the initial sediment-crust interface, if any sedimentation
if exist('surf.0','file') == 2
    read_surf=fopen('surf.0','r',plat);
    V=fread(read_surf,'single');
    fclose(read_surf);
    
    V=V((istp-1)*2000+1:(istp-1)*2000+2000);
    x_surf=V(1:1000);
    y_surf=V(1001:2000);
    plot(x_surf/1000,y_surf/1000,'y','linewidth',2);
end

%==========================================================================
%Labels
set(gca,'layer','top','box','on')
xlabel('Across-Axis Distance (km)','FontSize',15)
ylabel('Depth (km)','FontSize',15)
%title(['Time = ',num2str(t(kt)/3600/24/365/1000),' kyrs'])%      Frame ',num2str(istp)])
axis([xmin xmax zmin zmax])
xticks(-45:15:45);
% set(gca,'XTick',-45:15:45)
axis equal

ax = gca;
ax.XAxis.FontSize = 15;
ax.YAxis.FontSize = 15;

%==========================================================================
%Saving
if save
    print([field,num2str(istp)],'-dpng') ;
end

end